<?php

 /**
 * A class containing utility functions to speed up gotext's service development
 *
 * @category    goText Services' Support Classes
 * @package     goText
 * @author      Fulvio 'Zydio' Sciarretta <zydio@users.sf.net>
 * @version     Alpha 2
 * @copyright   Zydio (c) 2007-2009
 * @license     GNU GPL v2.0 http://www.gnu.org/licenses/gpl-2.0.html
 * @access public
 */
class goText_Utils {
    /**
     * Constructor (private because this class doesn't need to be instantiated)
     *
     * @access private
     */
    private function __construct(){  }

    /**
    * This text will be replaced in split parts with the current part number
    */
    const SPLIT_CURRENT_PART = "!#p!";
    /**
    * This text will be replaced in split parts with the total number of parts
    */
    const SPLIT_TOTAL_PARTS = "!#n!";

    /**
     * Splits a message in parts of the specified length
     *
     * @param string $text          Message text to be splitted
     * @param int $singleSmsLen     Lenght of the single message
     * @param string $headingText   Heading to put in front of every splitted message
     * @param string $footerText    Heading to put after every splitted message
     * @param boolean $keepFullWords    Split only on space or exactly at $singleSmsLen
     * @return array    Array of messages resulting from the split
     */
    public static function splitSms($text, $singleSmsLen, $headingText = "", $footerText = "", $keepFullWords = false) {
        $result = array();
        while($text != "") {
            $space="";
            $delta=-1;
            // Cerco la posizione del primo spazio vicino al limite di caratteri per pagina
            //  in modo da dividere le pagine senza spezzare le parole
            if ($keepFullWords) {
                do {
                    $delta++;
                    $space = substr($text, $singleSmsLen+$delta-1, 1);
                } while ((strcmp($space," ") != 0) && (($singleSmsLen+$delta) <= strlen($text)));
                // Se il testo non contiene spazi lo taglio a $singleSmsLen preciso
                if (($singleSmsLen+$delta) == strlen($text)) {
                    $delta = 0;
                }
            } else {
                $delta = 0;
            }

            $result[] = substr($text, 0, $singleSmsLen+$delta);
            $text = substr($text, $singleSmsLen+$delta);
            if ($text != "" || count($result) > 1) {
                $result[count($result)-1] = $headingText. $result[count($result)-1].$footerText;
            }
        }
        for($i = 0; $i <count($result); $i++ ) {
            $result[$i] = str_replace(self::SPLIT_CURRENT_PART, $i+1, $result[$i]);
            $result[$i] = str_replace(self::SPLIT_TOTAL_PARTS, count($result), $result[$i]);
        }
        return $result;
    }

    private static function _initRand()
    {
        static $randCalled = FALSE;
        if (!$randCalled)
        {
            srand( (double)microtime() * 1000000);
            $randCalled = TRUE;
        }
    }
    public static function randNum($low, $high)
    {
        self::_initRand();
        $rNum = rand($low, $high);
        return $rNum;
    }
    
    /**
     * Extracts from $startstr to $endstr in $where
     *
     * @param string $where       
     * @param string $startstr    
     * @param string $endstr      
     * @return string    
     */
    public static function str_extract($where, $startstr, $endstr) {
        $tmp = explode($startstr, $where);
        $tmp = explode($endstr, $tmp[1]);
        $result = $tmp[0];
        
        return $result;
    }
    
    /**
     * Removes null and duplicate values from $array and reassings keys
     * Called by rcpt2Array()
     * 
     * @param array $array    
     * @return array    
     */
    private static function _arrayClean($array) {
        $result = array_unique(array_filter($array));
        if (!count($result)) return $result;
        else return array_combine(range(0, count($result)-1), $result);
    }
    /**
    * Called by rcpt2Array() to store errors
    * 
    * $_RCPT2ARRAY_ERROR == 0 : void receipts string
    * $_RCPT2ARRAY_ERROR == 1 : not a number
    * $_RCPT2ARRAY_ERROR == 2 : not an email address
    * $_RCPT2ARRAY_ERROR == 3 : not a number nor an email address
    * $_RCPT2ARRAY_ERROR == 4 : not a valid number
    */
    private static $_RCPT2ARRAY_ERROR = 0;
    public static function rcpt2ArrayError() {
        return self::$_RCPT2ARRAY_ERROR;
    }
    /**
    * Called by rcpt2Array() to store wrong receipts
    */
    private static $_RCPT2ARRAY_WRONGRECEIPT = "";
    public static function rcpt2ArrayWrongReceipt() {
        return self::$_RCPT2ARRAY_WRONGRECEIPT;
    }
    /**
    * Used by rcpt2Array()
    */
    const NUMBER_MIN_LENGTH = 7;
    const NUMBER_MAX_LENGTH = 13;
    const LOCALNUMBER_MAX_LENGTH = 10;
    /**
     * Check consistency of receipts
     *
     * @param array $rcpt       String of receipts
     * @param array $validcc    CountryCodes accepted
     * @param int $defcc        Default CountryCode
     * @param int $in           Receipt type (1=number, 2=email, 3=both)
     * @return array    Array of valid receipts
     */
    public static function rcpt2Array($rcpt, $validcc, $defcc, $in = 1) {
        $arrayto = self::_arrayClean(explode(";",$rcpt));
        if (!count($arrayto)) return false;
        //check $in
        if ($in != 0 && $in != 1 && $in != 2) $in = 1;
        //check $validcc
        if (!is_array($validcc)) $validcc = array($validcc);
        //for all the receipts
        for ($i = 0; $i < count($arrayto); $i++) {
            //if it must be an email and it is not return false
            //else continue (do not check if it is a valid number)
            if ($in == 0) {
                    if (!preg_match('/^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$/i', $arrayto[$i], $matches)) {
                        self::$_RCPT2ARRAY_WRONGRECEIPT = $arrayto[$i];
                        self::$_RCPT2ARRAY_ERROR = 2;
                        return false;
                    }
                    else continue;
            }
            //if it is allowed to be an email and it is an email continue
            //else check if it is a valid number
            if ($in == 2 && preg_match('/^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$/i', $arrayto[$i], $matches)) continue;
            
            //get rid of starting + and 0 if any
            $arrayto[$i] = ltrim(trim($arrayto[$i]), "0+");
            //it must be a number
            if (!is_numeric($arrayto[$i])) {
                self::$_RCPT2ARRAY_WRONGRECEIPT = $arrayto[$i];
                if ($in == 2) self::$_RCPT2ARRAY_ERROR = 3;
                else self::$_RCPT2ARRAY_ERROR = 1;
                return false;
            }
            //check the length
            if (strlen($arrayto[$i]) > self::NUMBER_MAX_LENGTH || strlen($arrayto[$i]) < self::NUMBER_MIN_LENGTH) {
                self::$_RCPT2ARRAY_WRONGRECEIPT = $arrayto[$i];
                self::$_RCPT2ARRAY_ERROR = 4;
                return false;
            }
            //then check the prefix
            $invalidcc = true;
            //look for a $validcc
            foreach ($validcc as $cc) {
                if (substr($arrayto[$i], 0, strlen($cc)) == $cc) {
                    $invalidcc = false;
                    break;
                }
            }
            //use $defcc at worst, if $arrayto[$i] is not too long
            if ($invalidcc && strlen($arrayto[$i]) <= self::LOCALNUMBER_MAX_LENGTH) {
                $arrayto[$i] = $defcc.$arrayto[$i];
                $invalidcc = false;
            }
            //if number is not valid end
            if ($invalidcc) {
                self::$_RCPT2ARRAY_WRONGRECEIPT = $arrayto[$i];
                self::$_RCPT2ARRAY_ERROR = 4;
                return false;
            }
        }//end for
        return self::_arrayClean($arrayto);
    }

}

?>
